//  ===================================================================
//
//  Program:  commonlib
//  Module:   svddcmp.h
//  Language: C++
//  Date:     February 2005
//  Version:  1.0.0
//
//  Copyright (c) 2005
//
//  Marcelo Siqueira
//
//  Graduate Student
//  Department of Computer and Information Science
//  Moore School Building
//  200 South 33rd Street, Suite #62
//  Philadelphia, PA 19104-6389
//  Phone No.: (215) 898-5879
//
//  marcelos@seas.upenn.edu
//
// This program  is distributed  WITHOUT ANY WARRANTY,  and it  may be
// freely redistributed under the condition that the copyright notices
// are  not  removed,  and  no  compensation  is  received.   Private,
// research, and institutional use  is free. Distribution of this code
// as  part of  a  commercial  system is  permissible  ONLY BY  DIRECT
// ARRANGEMENT WITH THE AUTHOR.
//
//  ===================================================================

#ifndef __svddcmp_h
#define __svddcmp_h

#include "3dvector.h"


namespace common {


  // -----------------------------------------------------------------
  // class tSVDdcmp
  // --------------

  class tSVDdcmp {
  public:
    // constructors
    tSVDdcmp();
    tSVDdcmp(double**, int, int);
    tSVDdcmp(const tSVDdcmp&);

    // destructor
    ~tSVDdcmp();

    // get methods
    double matA(int, int) const throw (tExceptionObject);
    double matV(int, int) const throw (tExceptionObject);
    double vecW(int) const throw (tExceptionObject);

    // linear system solver
    void lsqSolve(double*, double*) const;

  private:
    //
    // private methods
    //
    void allocate();
    void deallocate();
    void decomp() const throw (tExceptionObject);
    void bksb(double*, double*) const;
    double pythag(double, double) const;
    double sign(const double&, const double&) const;

    //
    // private attributes
    //
    double** m_matA;
    double** m_matV;
    double*  m_vecW;
    int m_dimm;
    int m_dimn;
  };


}       // end of namespace common


#endif   // __svddcmp_h
