/** 
 * @class dcel::tCPAttrib
 *  
 * @brief A class representing a set of attributes for a cell complex.
 *
 * @author
 * Marcelo Ferreira Siqueira @n
 * Universidade Federal de Mato Grosso do Sul, @n
 * Departamento de Computacao e Estatistica, @n
 * marcelo at dct (dot) ufms (dot) br
 *
 * @version 1.0
 * @date Abril 2008
 *
 * @attention This program is distributed  WITHOUT ANY WARRANTY,  and
 *            it may be freely redistributed under the condition that
 *            the copyright notices are  not  removed,  and  no
 *            compensation  is  received. Private, research, and
 *            institutional use  is free. Distribution of this code as
 *            part of a commercial system is permissible ONLY BY DIRECT
 *            ARRANGEMENT WITH THE AUTHOR.
 */


#ifndef CPATTRIB_H_
#define CPATTRIB_H_


#include "smartptr.h"
#include "dataobj.h"


namespace dcel {


  /**
   * Forward definition of classes.
   */
  class tComplex;


  class tCPAttrib : public tDataObject {
  public:
    /**
     * Destructor.
     */
    virtual ~tCPAttrib() {}


    /**
     * Returns a pointer to the owner of this attribute.
     *
     * @return A pointer to a complex.
     * 
     */
    tSmartPtr<tComplex> get_owner() const {
      return m_owner;
    }


    /**
     * Makes tComplex a friend class.
     *
     */
    friend class tComplex;



  protected:
    /**
     * Default constructor.
     */
    tCPAttrib() {}


    /**
     * Assigns a given pointer to the owner attribute of this class.
     *
     * @param c: pointer to a complex.
     * 
     */
    void set_owner(const tSmartPtr<tComplex>& c) {
      m_owner = c;
    }


    /** 
     * Pointer to the owner of this attribute. 
     */
    tSmartPtr<tComplex> m_owner;


  };


}


#endif   // CPATTRIB_H_
