/** 
 * @class dcel::tFCAttrib
 *  
 * @brief A class representing a set of face attributes.
 *
 * @author
 * Marcelo Ferreira Siqueira @n
 * Universidade Federal de Mato Grosso do Sul, @n
 * Departamento de Computacao e Estatistica, @n
 * marcelo at dct (dot) ufms (dot) br
 *
 * @version 1.0
 * @date Abril 2008
 *
 * @attention This program is distributed  WITHOUT ANY WARRANTY,  and
 *            it may be freely redistributed under the condition that
 *            the copyright notices are  not  removed,  and  no
 *            compensation  is  received. Private, research, and
 *            institutional use  is free. Distribution of this code as
 *            part of a commercial system is permissible ONLY BY DIRECT
 *            ARRANGEMENT WITH THE AUTHOR.
 */


#ifndef FCATTRIB_H_
#define FCATTRIB_H_


#include "smartptr.h"
#include "dataobj.h"


namespace dcel {


  /**
   * Forward definition of classes.
   */
  class tComplex;
  class tFace;

  
  class tFCAttrib : public tDataObject {
  public:
    /**
     * Destructor.
     */
    virtual ~tFCAttrib() {}


    /**
     * Returns a pointer to the owner of this attribute.
     *
     * @return A pointer to a face.
     * 
     */
    tSmartPtr<tFace> get_owner() const {
      return m_owner;
    }


    /**
     * Makes tComplex a friend class.
     *
     */
    friend class tComplex;


  protected:
    /**
     * Default constructor.
     */
    tFCAttrib() {}


    /**
     * Assigns a given pointer to the owner attribute of this class.
     *
     * @param f: pointer to a face.
     * 
     */
    void set_owner(const tSmartPtr<tFace>& f) {
      m_owner = f;
    }


    /** 
     * Pointer to the owner of this attribute. 
     */
    tSmartPtr<tFace> m_owner;

  };


}


#endif   // FCATTRIB_H_
