/** 
 * @class dcel::tPoint
 *  
 * @brief A 3D point class.
 *
 * @author
 * Marcelo Ferreira Siqueira @n
 * Universidade Federal de Mato Grosso do Sul, @n
 * Departamento de Computacao e Estatistica, @n
 * marcelo at dct (dot) ufms (dot) br
 *
 * @version 1.0
 * @date Abril 2008
 *
 * @attention This program is distributed  WITHOUT ANY WARRANTY,  and
 *            it may be freely redistributed under the condition that
 *            the copyright notices are  not  removed,  and  no
 *            compensation  is  received. Private, research, and
 *            institutional use  is free. Distribution of this code as
 *            part of a commercial system is permissible ONLY BY DIRECT
 *            ARRANGEMENT WITH THE AUTHOR.
 */

#ifndef POINT_H_
#define POINT_H_


namespace dcel {


  /**
   * Class tPoint: implements a point in R2.
   *
   */ 
  class tPoint { 
  public:
    /**
     * Default Constructor.
     */
    tPoint() : m_x(0), m_y(0), m_z(0) {}


    /**
     * Constructor that receives the coordinates of the point.
     *
     * @param x: X coordinate of the point.
     * @param y: Y coordinate of the point.
     * @param z: Z coordinate of the point.
     *
     */
    tPoint(double x, double y, double z) : m_x(x), m_y(y), m_z(z) {}


    /**
     * Copy Constructor
     *
     * @param p: A 3D point.
     *
     */
      tPoint(const tPoint& p) : m_x(p.m_x), m_y(p.m_y), m_z(p.m_z)
    {};


    /**
     * Destructor
     *
     */
    ~tPoint() {}

  
    /**
     * Returns the X coordinate of the point.
     *
     * @return The value of the X coordinate of this point.
     *
     */
    double x() const { return m_x; }


    /**
     * Returns the Y coordinate of the point.
     *
     * @return The value of the Y coordinate of this point.
     *
     */
    double y() const { return m_y; }


    /**
     * Returns the Z coordinate of the point.
     *
     * @return The value of the Z coordinate of this point.
     *
     */
    double z() const { return m_z; }


    /** 
     * Overload operator to test equality.
     *
     * @param  p:  An object  of  @link  cqmesh::tPoint this  @endlink
     * class.
     *
     * @return @b True if the both  objects have the same coordinates.
     *
     */
     bool operator==(const tPoint& p) const {
       return (m_x == p.m_x) && (m_y == p.m_y) && (m_z == p.m_z);
     }


     /** 
      * Overload assignment operator.
      *
      * @param  p: An  object  of @link  cqmesh::tPoint this  @endlink
      * class.
      *
      * @return This object.
      *
      */
     tPoint& operator=(const tPoint& p) {
       m_x = p.m_x;
       m_y = p.m_y;
       m_z = p.m_z;

      return *this;
     }

  
  private:
    /**
     * X coordinate of the point.
     *
     */
     double m_x;


    /**
     * Y coordinate of the point.
     *
     */
     double m_y;

    /**
     * Z coordinate of the point.
     *
     */
     double m_z;

  };


}


#endif     // POINT_H_
