//  ===================================================================
//
//  Program:  OFFLIB
//  Module:   lexer.h
//  Language: C++
//  Date:     February 2008
//  Version:  1.0.0
//
//  Copyright (c) 2008
//
//  Marcelo Siqueira
//
//  Departamento de Computao e Estatstica
//  Universidade Federal de Mato Grosso do Sul
//  Av. Costa e Silva, S/N, Campus Universitria
//  Campo Grande, MS, Brasil, 79090-700
//  Phone No.: +55 67 3345 7458
//             +55 67 3345 7515
//
//  marcelo@dct.ufms.br
//
// This program  is distributed  WITHOUT ANY WARRANTY,  and it  may be
// freely redistributed under the condition that the copyright notices
// are  not  removed,  and  no  compensation  is  received.   Private,
// research, and institutional use  is free. Distribution of this code
// as  part of  a  commercial  system is  permissible  ONLY BY  DIRECT
// ARRANGEMENT WITH THE AUTHOR.
//
//  ===================================================================

#ifndef __lexer_h
#define __lexer_h

#include <iostream>
#include <string>

#include "exceptionobj.h"


namespace offlib {


  // -----------------------------------------------------------------
  // class tLexer
  //
  // This class implements a lexical analyzer to recognize the strings
  // and numbers in a *.OFF file.
  //
  // -----------------------------------------------------------------
  class tLexer {
  public:
    // constructor
    tLexer (std::istream&);

    // public methods
    bool get_str(std::string&) throw (tExceptionObject);
    bool get_int(int&) throw (tExceptionObject);
    bool get_uint(unsigned&) throw (tExceptionObject);
    bool get_double(double&) throw (tExceptionObject);
    int linecount() const { return m_lcount; }
    int charcount() const { return m_ccount; }

  private:
    // private methods
    void increment_line();
    void increment_char();
    void decrement_char();
    bool skip_space();
    void reset_char_counter() { m_ccount = 0; }

    // private attributes
    std::istream& m_realstr;
    int m_lcount;
    int m_ccount;
  };


}       // end of namespace offlib


#endif
