//  ===================================================================
//
//  Program:  OFFLIB
//  Module:   reader.h
//  Language: C++
//  Date:     February 2008
//  Version:  1.0.0
//
//  Copyright (c) 2008
//
//  Marcelo Siqueira
//
//  Departamento de Computao e Estatstica
//  Universidade Federal de Mato Grosso do Sul
//  Av. Costa e Silva, S/N, Campus Universitria
//  Campo Grande, MS, Brasil, 79090-700
//  Phone No.: +55 67 3345 7458
//             +55 67 3345 7515
//
//  marcelo@dct.ufms.br
//
// This program  is distributed  WITHOUT ANY WARRANTY,  and it  may be
// freely redistributed under the condition that the copyright notices
// are  not  removed,  and  no  compensation  is  received.   Private,
// research, and institutional use  is free. Distribution of this code
// as  part of  a  commercial  system is  permissible  ONLY BY  DIRECT
// ARRANGEMENT WITH THE AUTHOR.
//
//  ===================================================================

#ifndef __reader_h
#define __reader_h

#include "exceptionobj.h"
#include "lexer.h"

#include <fstream>
#include <string>


namespace offlib {


  // -----------------------------------------------------------------
  // class tReader
  //
  // This  class  implements  a   reader  of  OFF  files  representing
  // triangular meshes.
  //
  // -----------------------------------------------------------------

  class tReader {
  public:
    // constructors
    tReader(const std::string& fn) : m_is(0)
    {
      m_fname = fn + std::string(".off");
    }

    ~tReader() { if (m_is != 0) delete m_is; }

    // reads a *.OFF file
    void read(unsigned&, double*&, unsigned&, unsigned*&) 
      throw (tExceptionObject);

  private:
    void read_header(unsigned&, unsigned&) throw (tExceptionObject);
    void read_verts(unsigned, double*&) throw (tExceptionObject);
    void read_faces(unsigned, unsigned, unsigned*&) 
      throw (tExceptionObject);

    std::string m_fname;
    tLexer* m_is;
  };

}       // end of namespace offlib


#endif   // __reader_h
