//  ===================================================================
//
//  Program:  OFFLIB
//  Module:   writer.h
//  Language: C++
//  Date:     February 2008
//  Version:  1.0.0
//
//  Copyright (c) 2008
//
//  Marcelo Siqueira
//
//  Departamento de Computao e Estatstica
//  Universidade Federal de Mato Grosso do Sul
//  Av. Costa e Silva, S/N, Campus Universitria
//  Campo Grande, MS, Brasil, 79090-700
//  Phone No.: +55 67 3345 7458
//             +55 67 3345 7515
//
//  marcelo@dct.ufms.br
//
// This program  is distributed  WITHOUT ANY WARRANTY,  and it  may be
// freely redistributed under the condition that the copyright notices
// are  not  removed,  and  no  compensation  is  received.   Private,
// research, and institutional use  is free. Distribution of this code
// as  part of  a  commercial  system is  permissible  ONLY BY  DIRECT
// ARRANGEMENT WITH THE AUTHOR.
//
//  ===================================================================

#ifndef __writer_h
#define __writer_h

#include <string>
#include <fstream>

#include "exceptionobj.h"


namespace offlib {

  
  // -----------------------------------------------------------------
  // class tWriter
  //
  // This  class  writes  the  vertex,  edge, and  face  files  of   a
  // triangulated surface.
  //
  // -----------------------------------------------------------------
  class tWriter {
  public:
    // constructors
    tWriter(const std::string&) throw (tExceptionObject);
    tWriter(const tWriter&);
    
    // destructor
    ~tWriter();
    
    // public methods
    void write(unsigned, const double*, unsigned, const unsigned*) 
      throw (tExceptionObject);

    void write_qm(unsigned, const double*, unsigned, const unsigned*) 
      throw (tExceptionObject);
    
  private:
    void write_header(unsigned, unsigned);
    void write_verts(unsigned, const double*) 
      throw (tExceptionObject);
    void write_faces(unsigned, const unsigned*) 
      throw (tExceptionObject);
    void write_faces_qm(unsigned, const unsigned*) 
      throw (tExceptionObject);
    
    // image file name
    std::string m_fname;
    std::fstream m_fs;
  };


}   // end namespace offlib


#endif    // __writer_h
