/** 
 * @class pps::tEvalStruct
 *  
 * @brief A  class implementing a Loop's subdivision surface exact evaluator.
 *
 * @author
 * Marcelo Ferreira Siqueira @n
 * Universidade Federal de Mato Grosso do Sul, @n
 * Departamento de Computacao e Estatistica, @n
 * marcelo at dct (dot) ufms (dot) br
 *
 * @version 1.0
 * @date May 2008
 *
 * @attention This program is distributed  WITHOUT ANY WARRANTY,  and
 *            it may be freely redistributed under the condition that
 *            the copyright notices are  not  removed,  and  no
 *            compensation  is  received. Private, research, and
 *            institutional use  is free. Distribution of this code as
 *            part of a commercial system is permissible ONLY BY DIRECT
 *            ARRANGEMENT WITH THE AUTHOR.
 */


#ifndef LOOP_EVAL_H_
#define LOOP_EVAL_H_

#include <stdio.h>
#include <stdlib.h>
#include <cmath>


/*********************************************************************
 *
 * This  code  was borrowed  from  Jos  Stam,  who created  the  first
 * approach for exact evaluation of Catmull-Clark and Loop subdivision
 * surfaces:
 *
 * Stam, Jos. "Exact  Evaluation of Catmull-Clark Subdivision Surfaces
 * at Arbitrary Parameter  Values".  In Computer Graphics Proceedings,
 * Annual Conference Series, 1998, pages 395-404, July 1998.
 *
 *
 * The  class tEvalStruct  is basically  a C++  wrapper class  for the
 * original C code by Stam, who kindly provided me with his original C
 * implementation.  If  you find any  problems with this  code, please
 * email ME  and not  him (as I  am the  one responsible for  the code
 * below):
 *
 * mfsiqueira at gmail dot com
 *
 ********************************************************************/


namespace pps {


  class tEvalStruct {
  public:

    typedef enum { EVAL_VALUE=1 , EVAL_DER=2, EVAL_DER2=3 } EVALTYPE ;

    struct EVALSTRUCT
    {
      double * val;
      double * vecI;
      double ** Phi;
    };

    tEvalStruct( const char* );

    ~tEvalStruct();

    double* proj_control_points( double* , int ) const ;

    void eval_surf( EVALTYPE , double* , int , double , double , double[3][3] ) const ;

    void printout() const ;

    int get_max_valence() const { return Nmax; }

  private:
    int IX( int i, int j, int n ) const { return i + n * j ; }

    double* get_mem( int ) const ;

    void eval_T_spline ( double* , double , double ) const ;
    void eval_T_spline_v( double* , double , double ) const ;
    void eval_T_spline_w( double* , double , double ) const ;
    void eval_T_spline_vv( double* , double , double ) const ;
    void eval_T_spline_vw( double* , double , double ) const ;
    void eval_T_spline_ww( double* , double , double ) const ;

    const double BIG_ONE ;
    
    EVALSTRUCT** ev_struct;
    int Nmax;

  };

}

#endif
